﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ESRI.ArcGIS.SOAP;
using System.Net;
using System.Timers;

namespace SOAPSecurityWpf
{
    public partial class Window1 : Window
    {
        private MapServerProxy _mapservice;
        private DateTime _endTime;
        private int _timeout = 1; //minutes - defines token service timeout

        // Token test
        private string _serviceurl = "http://net931/arcgistoken/services/USA_Data/MapServer";
        private string _username = "test";
        private string _password = "test.test";
        private string _domain = "";
        
        // Windows auth test
        //private string _serviceurl = "http://net931/arcgis/services/USA_Data/MapServer";
        //private string _username = "rex";
        //private string _password = "rex";
        //private string _domain = "net931";

        System.Windows.Threading.DispatcherTimer _updateTimer;

        public Window1()
        {
            InitializeComponent();
        }

        private MapServerProxy MapServiceProxy
        {
            get
            {
                if (_mapservice == null)
                {
                    _mapservice = new MapServerProxy();
                    _mapservice.Url = _serviceurl;
                }

                if (_mapservice.Credentials != null)
                {
                    return _mapservice;
                }
                else if (DateTime.Now.CompareTo(_endTime) >= 0)
                {
                    _endTime = DateTime.Now.AddMinutes(_timeout);
                    _mapservice = AGSSOAPUtility.AuthenticateProxy.Authenticate
                        (_mapservice, _username, _password, _domain, _timeout) as MapServerProxy;
                }

                return _mapservice;
            }
        }

        int secondsSinceRequest;
        private void MyButton_Click(object sender, RoutedEventArgs rea)
        {
            // Show time (seconds) since last request.  Use to test token timeout.
            if (_updateTimer == null)
                _updateTimer = new System.Windows.Threading.DispatcherTimer();
            else
                _updateTimer.Stop();

            _updateTimer.Interval = TimeSpan.FromMilliseconds(1000);
            _updateTimer.Tick += (evtsender, a) =>
            {
                secondsSinceRequest = (int) DateTime.Now.Subtract((DateTime)_updateTimer.Tag).TotalSeconds;
                TimerText.Text = "Seconds since last request: " + secondsSinceRequest.ToString();                
            };
            _updateTimer.Tag = DateTime.Now;
            _updateTimer.Start();  

            // Initiate request to generate map image
            ShowSimpleMapImage();
        }

        private void ShowSimpleMapImage()
        {
            MapServerInfo mapServerInfo = MapServiceProxy.GetServerInfo(MapServiceProxy.GetDefaultMapName());
            MapDescription mapDescription = mapServerInfo.DefaultMapDescription;

            ImageType imageType = new ImageType();
            imageType.ImageFormat = esriImageFormat.esriImageJPG;
            imageType.ImageReturnType = esriImageReturnType.esriImageReturnURL;

            ImageDisplay imageDisplay = new ImageDisplay();
            imageDisplay.ImageHeight = (int)MyImage.Height;
            imageDisplay.ImageWidth = (int)MyImage.Width;
            imageDisplay.ImageDPI = 96;

            ImageDescription imageDescription = new ImageDescription();
            imageDescription.ImageDisplay = imageDisplay;
            imageDescription.ImageType = imageType;

            MapImage mapImage = MapServiceProxy.ExportMapImage(mapDescription, imageDescription);

            MyImage.Source = new BitmapImage(new Uri(mapImage.ImageURL, UriKind.Absolute));
        }

        private void MyWindow_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            if (e.PreviousSize.Width == 0 && e.PreviousSize.Height == 0)
            {
                MyImage.Width = e.NewSize.Width;
                MyImage.Height = e.NewSize.Height;
            }
        }
    }

    public class ExtendMapServerProxy : MapServerProxy
    {
        System.Net.ICredentials _customCredentials;
        string _referer;

        public System.Net.ICredentials CustomCredentials
        {
            get { return _customCredentials; }
            set { _customCredentials = value; }
        }

        public string Referer
        {
            get { return _referer; }
            set { _referer = value; }
        }

        protected override System.Net.WebRequest GetWebRequest(Uri uri)
        {
            HttpWebRequest request = (HttpWebRequest)base.GetWebRequest(uri);
            if (_customCredentials != null)
                request.Credentials = _customCredentials;
            request.Referer = _referer;
            return request;
        }
    }
}
